addpath(genpath(pwd));

% Read the shape, represented as a triangle mesh.
fprintf('reading the shape...');
S1 = read_off_shape('mesh004.off');
C1 = [S1.surface.X S1.surface.Y S1.surface.Z];
S1.surface.X = C1(:,2);
S1.surface.Y = C1(:,3);
S1.surface.Z = C1(:,1);

fprintf('done\n');

% Compute the cotangent weight discretization of the Laplacian.
fprintf('computing the Laplacian...');
A = vertexAreas(C1,S1.surface.TRIV);
W = cotWeights(C1,S1.surface.TRIV);
A = A / sum(A);

nv = S1.surface.nv;
% Compute the eigenvalues/vectors of the Laplacian.
[e, v] = eigs(W,spdiags(A,0,nv,nv),30,-1e-6);

L1.A = spdiags(A,0,nv,nv);
L1.evecs = e;
L1.evals = diag(v);
fprintf('done\n');

%%
% Sample function -- in this case it is the Heat Kernel Signature for time
% value t = 0.01
[f,f_c] = hks(L1,0.01);
f = f-min(f);

%%
% Compute the initial persistence diagram.
subplot(2,2,1);
[birth_ids, death_ids] = paired_mesh_persistence(S1.surface, f);

plot_persistence(f, birth_ids, death_ids);
title('Persistence diagram before optimization');

subplot(2,2,2);
plot_persistence_function(S1, f, birth_ids, death_ids);
title('Function before optimization');

%%
% Optimize the persistence diagram
% note that we optimize the coefficients of the target function in the
% basis given by the Laplacian eigenfunctions. This both helps to reduce
% the number of unknowns and also strongly regularizes the problem.
fprintf('Optimizing persistence...');tic;
funProj = @(F) [0; F(2:end)];
options.verbose = 1;
[c, ~] = minConf_PQN(@(F) persistence_objective(S1.surface, L1.evecs, F), f_c, funProj, options);
fprintf('done\n');toc;

%%
% Plot the final results.
f = L1.evecs * c;
f = f-min(f);
subplot(2,2,3);
[birth_ids, death_ids] = paired_mesh_persistence(S1.surface, f);

plot_persistence(f, birth_ids, death_ids);
title('Persistence diagram after optimization');

subplot(2,2,4);
plot_persistence_function(S1, f, birth_ids, death_ids);
title('Function after optimization');

